
// (c) COPYRIGHT URI/MIT 1995-1996
// Please read the full copyright statement in the file COPYRIGH.  
//
// Authors:
//      jhrg,jimg       James Gallagher (jgallagher@gso.uri.edu)

// Implementation of Function.
//
// jhrg 9/15/94

// $Log: Function.cc,v $
// Revision 1.33  1998/03/17 17:31:01  jimg
// Deprecated.
//
// Revision 1.32  1997/09/22 23:02:26  jimg
// Added DDS * to deserialize parameters.
//
// Revision 1.31  1997/03/08 19:01:59  jimg
// Changed default param to check_semantics() from  to String()
// and removed the default from the argument list in the mfunc definition
//
// Revision 1.30  1997/02/28 01:27:56  jimg
// Changed check_semantics() so that it now returns error messages in a String
// object (passed by reference).
//
// Revision 1.29  1997/02/10 02:32:38  jimg
// Added assert statements for pointers
//
// Revision 1.28  1996/09/16 18:08:59  jimg
// Fixed var(const String name) so that it would correctly descend names of the
// form <base>.<name> where <name> may itself contain `dots'.
//
// Revision 1.27  1996/08/13 18:26:28  jimg
// Added cast to void of values calculated in the print_val() for loops. This to
// eliminate a gcc warning.
//
// Revision 1.26  1996/06/04 21:33:28  jimg
// Multiple connections are now possible. It is now possible to open several
// URLs at the same time and read from them in a round-robin fashion. To do
// this I added data source and sink parameters to the serialize and
// deserialize mfuncs. Connect was also modified so that it manages the data
// source `object' (which is just an XDR pointer).
//
// Revision 1.25  1996/05/31 23:29:43  jimg
// Updated copyright notice.
//
// Revision 1.24  1996/05/22 18:05:11  jimg
// Merged files from the old netio directory into the dap directory.
// Removed the errmsg library from the software.
//
// Revision 1.23  1996/05/16 22:49:45  jimg
// Dan's changes for version 2.0. Added a parameter to read that returns
// an error code so that EOF can be distinguished from an actual error when
// reading sequences. This *may* be replaced by an error member function
// in the future.
//
// Revision 1.22  1996/05/14 15:38:24  jimg
// These changes have already been checked in once before. However, I
// corrupted the source repository and restored it from a 5/9/96 backup
// tape. The previous version's log entry should cover the changes.
//
// Revision 1.21  1996/04/05 00:21:31  jimg
// Compiled with g++ -Wall and fixed various warnings.
//
// Revision 1.20  1996/04/04 18:34:30  jimg
// Merged changes from version 1.1.1.
//
// Revision 1.19  1996/03/05 18:15:12  jimg
// Added ce_eval to serailize member function.
// Fixed serialize bug where expunge() might not be called even if the the
// FLUSH parameter is true and the member function completes.
// Fixed various instances of variables defined within FOR statements.
//
// Revision 1.18  1996/02/02 00:31:04  jimg
// Merge changes for DODS-1.1.0 into DODS-2.x
//
// Revision 1.17  1995/12/09  01:06:43  jimg
// Added changes so that relational operators will work properly for all the
// datatypes (including Sequences). The relational ops are evaluated in
// DDS::eval_constraint() after being parsed by DDS::parse_constraint().
//
// Revision 1.16  1995/12/06  21:56:21  jimg
// Added `constrained' flag to print_decl.
// Removed third parameter of read.
// Modified print_decl() to print only those parts of a dataset that are
// selected when `constrained' is true.
//
// Revision 1.15  1995/10/23  23:20:51  jimg
// Added _send_p and _read_p fields (and their accessors) along with the
// virtual mfuncs set_send_p() and set_read_p().
//
// Revision 1.14  1995/08/26  00:30:16  jimg
// Fixed return types of indep_var() and dep_var().
// Removed old code enclosed in #ifdef NEVER #endif.
//
// Revision 1.13  1995/08/23  00:11:10  jimg
// Changed old, deprecated member functions to new ones.
// Switched from String representation of type to enum.
//
// Revision 1.12.2.1  1995/09/14 20:58:18  jimg
// Moved some loop index variables out of the loop statement.
//
// Revision 1.12  1995/07/09  21:28:58  jimg
// Added copyright notice.
//
// Revision 1.11  1995/05/10  15:33:58  jimg
// Failed to change `config.h' to `config_dap.h' in these files.
//
// Revision 1.10  1995/05/10  13:45:15  jimg
// Changed the name of the configuration header file from `config.h' to
// `config_dap.h' so that other libraries could have header files which were
// installed in the DODS include directory without overwriting this one. Each
// config header should follow the convention config_<name>.h.
//
// Revision 1.9  1995/03/16  17:29:08  jimg
// Added include config_dap.h to top of include list.
// Added TRACE_NEW switched dbnew includes.
// Fixed bug in read_val() where **val was passed incorrectly to
// subordinate read_val() calls.
//
// Revision 1.8  1995/03/04  14:34:44  jimg
// Major modifications to the transmission and representation of values:
// Added card() virtual function which is true for classes that
// contain cardinal types (byte, int float, string).
// Changed the representation of Str from the C rep to a C++
// class represenation.
// Chnaged read_val and store_val so that they take and return
// types that are stored by the object (e.g., inthe case of Str
// an URL, read_val returns a C++ String object).
// Modified Array representations so that arrays of card()
// objects are just that - no more storing strings, ... as
// C would store them.
// Arrays of non cardinal types are arrays of the DODS objects (e.g.,
// an array of a structure is represented as an array of Structure
// objects).
//
// Revision 1.7  1995/02/10  02:23:05  jimg
// Added DBMALLOC includes and switch to code which uses malloc/free.
// Private and protected symbols now start with `_'.
// Added new accessors for name and type fields of BaseType; the old ones
// will be removed in a future release.
// Added the store_val() mfunc. It stores the given value in the object's
// internal buffer.
// Made both List and Str handle their values via pointers to memory.
// Fixed read_val().
// Made serialize/deserialize handle all malloc/free calls (even in those
// cases where xdr initiates the allocation).
// Fixed print_val().
//
// Revision 1.6  1995/01/19  20:05:25  jimg
// ptr_duplicate() mfunc is now abstract virtual.
// Array, ... Grid duplicate mfuncs were modified to take pointers, not
// referenves.
//
// Revision 1.5  1995/01/11  15:54:44  jimg
// Added modifications necessary for BaseType's static XDR pointers. This
// was mostly a name change from xdrin/out to _xdrin/out.
// Removed the two FILE pointers from ctors, since those are now set with
// functions which are friends of BaseType.
//
// Revision 1.4  1994/12/16  15:19:06  dan
// Modified class to inherit directly from class BaseType, allowing
// call to BaseType constructor during constructor call.
//
// Revision 1.3  1994/10/17  23:34:51  jimg
// Added code to print_decl so that variable declarations are pretty
// printed.
// Added private mfunc duplicate().
// Added ptr_duplicate().
// Added Copy ctor, dtor and operator=.
//
// Revision 1.2  1994/09/23  14:43:38  jimg
// Added mfunc check_semantics().
// Added sanity checking on the variable lists (are the lists empty?).
//

#ifdef _GNUG_
#pragma implementation
#endif

#include "config_dap.h"

#include <assert.h>

#include "Function.h"
#include "DDS.h"
#include "debug.h"
#include "util.h"

#ifdef TRACE_NEW
#include "trace_new.h"
#endif

void
Function::_duplicate(const Function &s)
{
    BaseType::_duplicate(s);

    Function &cs = (Function &)s;	// cast away const;

    Pix p;
    for (p = cs._dep_vars.first(); p; cs._dep_vars.next(p))
	_dep_vars.append(cs._dep_vars(p)->ptr_duplicate());

    for (p = cs._indep_vars.first(); p; cs._indep_vars.next(p))
	_indep_vars.append(cs._indep_vars(p)->ptr_duplicate());
}

Function::Function(const String &n) : BaseType(n, dods_function_c)
{
}

Function::Function(const Function &rhs)
{
    _duplicate(rhs);
}

Function::~Function()
{
    Pix p;
    for (p = _dep_vars.first(); p; _dep_vars.next(p))
	delete _dep_vars(p);

    for (p = _indep_vars.first(); p; _indep_vars.next(p))
	delete _indep_vars(p);
}

const Function &
Function::operator=(const Function &rhs)
{
    if (this == &rhs)
	return *this;

    _duplicate(rhs);

    return *this;
}

// Dummy mfunc until I remove this class.

int
Function::element_count(bool leaves)
{
    return 0;
}

void
Function::set_send_p(bool state)
{
    Pix p;
    for (p = _indep_vars.first(); p; _indep_vars.next(p))
	_indep_vars(p)->set_send_p(state);

    for (p = _dep_vars.first(); p; _dep_vars.next(p))
	_dep_vars(p)->set_send_p(state);

    BaseType::set_send_p(state);
}

void
Function::set_read_p(bool state)
{
    Pix p;
    for (p = _indep_vars.first(); p; _indep_vars.next(p))
	_indep_vars(p)->set_read_p(state);

    for (p = _dep_vars.first(); p; _dep_vars.next(p))
	_dep_vars(p)->set_read_p(state);

    BaseType::set_read_p(state);
}

// Assume that variable names within a function are unique. 

BaseType *
Function::var(const String &name)
{
    if (name.contains(".")) {
	String n = (String)name; // cast away const
	String aggregate = n.before(".");
	String field = n.from(".");
	field = field.after(".");

	BaseType *agg_ptr = var(aggregate);
	if (agg_ptr)
	    return agg_ptr->var(field);	// recurse
	else
	    return 0;		// qualified names must be *fully* qualified
    }
    else {
	Pix p;

	for (p = _indep_vars.first(); p; _indep_vars.next(p))
	    if (_indep_vars(p)->name() == name)
		return _indep_vars(p);

	for (p = _dep_vars.first(); p; _dep_vars.next(p))
	    if (_dep_vars(p)->name() == name)
		return _dep_vars(p);
    }

    return 0;
}

void 
Function::add_var(BaseType *bt, Part part)
{
    assert(bt);

    switch (part) {
      case independent:
	DBG(cerr << "Function::add_var: appending independent variable " \
	    << bt->name() << endl);
	_indep_vars.append(bt);
	return;
      case dependent:
	DBG(cerr << "Function::add_var: appending dependent variable " \
	    << bt->name() << endl);
	_dep_vars.append(bt);
	return;
      default:
	assert(false);
	cerr << "Function::add_var: Unknown function part" << endl;
	return;
    }
}

Pix 
Function::first_indep_var()
{
    return  _indep_vars.first();
}

void 
Function::next_indep_var(Pix &p)
{
    if (!_indep_vars.empty() && p)
	_indep_vars.next(p);
}

BaseType *
Function::indep_var(Pix p)
{
    if (!_indep_vars.empty() && p)
	return _indep_vars(p);
    else
	return 0;
}

Pix 
Function::first_dep_var()
{
    return  _dep_vars.first();
}

void 
Function::next_dep_var(Pix &p)
{
    if (!_dep_vars.empty() && p)
	_dep_vars.next(p);
}

BaseType *
Function::dep_var(Pix p)
{
    if (!_dep_vars.empty() && p)
	return _dep_vars(p);
    else
	return 0;
}

unsigned int 
Function::width()
{
    unsigned int sz = 0;
    
    Pix p;
    for(p = first_indep_var(); p; next_indep_var(p))
	sz += indep_var(p)->width();

    for(p = first_dep_var(); p; next_dep_var(p))
	sz += dep_var(p)->width();

    return sz;
}

bool 
Function::serialize(const String &dataset, DDS &dds, XDR *sink, 
		    bool ce_eval = true)
{
    bool status = true;
    int error = 0;

    while (status) {
	if (!read_p() && !read(dataset, error))
	    return false;

	if (ce_eval && !dds.eval_selection(dataset)) {
	    set_read_p(false);
	    continue;
	}

	Pix p;
	for (p = first_indep_var(); p; next_indep_var(p)) {
	    if (indep_var(p)->send_p() 
		&& !(status = indep_var(p)->serialize(dataset, dds, sink, 
						      false))) 
		break;
	}

	for (p = first_dep_var(); p; next_dep_var(p)) {
	    if (dep_var(p)->send_p() 
		&& !(status = dep_var(p)->serialize(dataset, dds, sink, 
						    false)))
		break;
	}

	set_read_p(false);
    }

    return status;
}

bool
Function::deserialize(XDR *source, DDS *dds, bool reuse = false)
{
    bool status = true;

    Pix p;
    for (p = first_indep_var(); p; next_indep_var(p)) {
	status = indep_var(p)->deserialize(source, dds, reuse);
	if (!status) 
	    return false;	// if the independent vars fail, return
    }

    for (p = first_dep_var(); p; next_dep_var(p)) {
	status = dep_var(p)->deserialize(source, dds, reuse);
	if (!status) 
	  break;		// failure here can be handled by a break
    }

    return status;
}

unsigned int
Function::val2buf(void *, bool)
{
    return sizeof(Function);
}

unsigned int
Function::buf2val(void **)
{
    return sizeof(Function);
}

void
Function::print_decl(ostream &os, String space, bool print_semi,
		     bool constraint_info, bool constrained)
{
    if (constrained && !send_p())
	return;

    os << space << type_name() << " {" << endl;

    os << space << " INDEPENDENT:" << endl;

    Pix p;
    for (p = _indep_vars.first(); p; _indep_vars.next(p))
	_indep_vars(p)->print_decl(os, space + "    ", true, constraint_info,
				   constrained);

    os << space << " DEPENDENT:" << endl;
    for (p = _dep_vars.first(); p; _dep_vars.next(p))
	_dep_vars(p)->print_decl(os, space + "    ", true, constraint_info,
				 constrained);

    os << space << "} " << name();

    if (constraint_info) {
	if (send_p())
	    cout << ": Send True";
	else
	    cout << ": Send False";
    }

    if (print_semi)
	os << ";" << endl;
}

void 
Function::print_val(ostream &os, String space, bool print_decl_p)
{
    if (print_decl_p) {
	print_decl(os, space, false);
	os << " = ";
    }

    os << "{ INDEPENDENT: ";

    Pix p;
    for (p = _indep_vars.first(); p; 
	 _indep_vars.next(p), (void)(p && os << ", "))
	_indep_vars(p)->print_val(os, "", false);

    os << " DEPENDENT: ";

    for (p = _dep_vars.first(); p; _dep_vars.next(p), (void)(p && os << ", "))
	_dep_vars(p)->print_val(os, "", false);

    os << " }";

    if (print_decl_p)
	os << ";" << endl;
}


bool
Function::check_semantics(String &msg, bool all = false)
{
    if (!BaseType::check_semantics(msg))
	return false;

    SLList<BaseTypePtr> fvars = _indep_vars;
    SLList<BaseTypePtr> fvars_tmp = _dep_vars; // join deletes its arg
    fvars.join(fvars_tmp);	// fvars is now a list of all the variables

    if (!unique_names(fvars, (const char *)name(), (const char *)type_name(),
		      msg))
	return false;

    if (all) 
	for (Pix p = fvars.first(); p; fvars.next(p))
	    if (!fvars(p)->check_semantics(msg, true))
		return false;

    return true;
}
