
// -*- C++ -*-

// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGH.  
//
// Authors:
//      jhrg,jimg       James Gallagher (jgallagher@gso.uri.edu)

// Interface to the Function ctor class. A function has two sets of variables,
// the independent and the dependent. The variables in a function must be
// unique to both parts (i.e., a variable named `i' in the independent
// section of a function means that no other variable in *either* that
// section or the dependent section may be named `i').
//
// jhrg 9/15/94

/* 
 * $Log: Function.h,v $
 * Revision 1.29  1998/03/17 17:31:20  jimg
 * Deprecated.
 *
 * Revision 1.28  1998/02/05 20:13:53  jimg
 * DODS now compiles with gcc 2.8.x
 *
 * Revision 1.27  1998/02/04 14:55:32  tom
 * Another draft of documentation.
 *
 * Revision 1.26  1998/01/12 14:27:58  tom
 * Second pass at class documentation.
 *
 * Revision 1.25  1997/12/18 15:06:11  tom
 * First draft of class documentation, entered in doc++ format,
 * in the comments
 *
 * Revision 1.24  1997/10/09 22:19:20  jimg
 * Resolved conflicts in merge of 2.14c to trunk.
 *
 * Revision 1.23  1997/08/11 18:19:16  jimg
 * Fixed comment leaders for new CVS version
 *
 * Revision 1.22  1997/03/08 19:02:00  jimg
 * Changed default param to check_semantics() from  to String()
 * and removed the default from the argument list in the mfunc definition
 *
 * Revision 1.21  1997/02/28 01:29:07  jimg
 * Changed check_semantics() so that it now returns error messages in a String
 * object (passed by reference).
 *
 * Revision 1.20  1996/06/04 21:33:30  jimg
 * Multiple connections are now possible. It is now possible to open several
 * URLs at the same time and read from them in a round-robin fashion. To do
 * this I added data source and sink parameters to the serialize and
 * deserialize mfuncs. Connect was also modified so that it manages the data
 * source `object' (which is just an XDR pointer).
 *
 * Revision 1.19  1996/05/31 23:29:44  jimg
 * Updated copyright notice.
 *
 * Revision 1.18  1996/05/16 22:50:01  jimg
 * Dan's changes for version 2.0. Added a parameter to read that returns
 * an error code so that EOF can be distinguished from an actual error when
 * reading sequences. This *may* be replaced by an error member function
 * in the future.
 *
 * Revision 1.17  1996/03/05 18:10:36  jimg
 * Added ce_eval to serailize member function.
 *
 * Revision 1.16  1995/12/09  01:06:44  jimg
 * Added changes so that relational operators will work properly for all the
 * datatypes (including Sequences). The relational ops are evaluated in
 * DDS::eval_constraint() after being parsed by DDS::parse_constraint().
 *
 * Revision 1.15  1995/12/06  21:56:23  jimg
 * Added `constrained' flag to print_decl.
 * Removed third parameter of read.
 * Modified print_decl() to print only those parts of a dataset that are
 * selected when `constrained' is true.
 *
 * Revision 1.14  1995/10/23  23:20:53  jimg
 * Added _send_p and _read_p fields (and their accessors) along with the
 * virtual mfuncs set_send_p() and set_read_p().
 *
 * Revision 1.13  1995/08/26  00:31:32  jimg
 * Removed code enclosed in #ifdef NEVER #endif.
 *
 * Revision 1.12  1995/08/22  23:48:16  jimg
 * Removed card() member function.
 * Removed old, deprecated member functions.
 * Changed the names of read_val and store_val to buf2val and val2buf.
 *
 * Revision 1.11  1995/05/10  13:45:16  jimg
 * Changed the name of the configuration header file from `config.h' to
 * `config_dap.h' so that other libraries could have header files which were
 * installed in the DODS include directory without overwriting this one. Each
 * config header should follow the convention config_<name>.h.
 *
 * Revision 1.10  1995/03/04  14:34:59  jimg
 * Major modifications to the transmission and representation of values:
 * Added card() virtual function which is true for classes that
 * contain cardinal types (byte, int float, string).
 * Changed the representation of Str from the C rep to a C++
 * class represenation.
 * Chnaged read_val and store_val so that they take and return
 * types that are stored by the object (e.g., inthe case of Str
 * an URL, read_val returns a C++ String object).
 * Modified Array representations so that arrays of card()
 * objects are just that - no more storing strings, ... as
 * C would store them.
 * Arrays of non cardinal types are arrays of the DODS objects (e.g.,
 * an array of a structure is represented as an array of Structure
 * objects).
 *
 * Revision 1.9  1995/02/10  02:23:03  jimg
 * Added DBMALLOC includes and switch to code which uses malloc/free.
 * Private and protected symbols now start with `_'.
 * Added new accessors for name and type fields of BaseType; the old ones
 * will be removed in a future release.
 * Added the store_val() mfunc. It stores the given value in the object's
 * internal buffer.
 * Made both List and Str handle their values via pointers to memory.
 * Fixed read_val().
 * Made serialize/deserialize handle all malloc/free calls (even in those
 * cases where xdr initiates the allocation).
 * Fixed print_val().
 *
 * Revision 1.8  1995/01/19  21:59:15  jimg
 * Added read_val from dummy_read.cc to the sample set of sub-class
 * implementations.
 * Changed the declaration of readVal in BaseType so that it names the
 * mfunc read_val (to be consistant with the other mfunc names).
 * Removed the unnecessary duplicate declaration of the abstract virtual
 * mfuncs read and (now) read_val from the classes Byte, ... Grid. The
 * declaration in BaseType is sufficient along with the decl and definition
 * in the *.cc,h files which contain the subclasses for Byte, ..., Grid.
 *
 * Revision 1.7  1995/01/18  18:39:32  dan
 * Declared member function 'readVal', defined in dummy_read.cc
 *
 * Revision 1.6  1995/01/11  15:54:45  jimg
 * Added modifications necessary for BaseType's static XDR pointers. This
 * was mostly a name change from xdrin/out to _xdrin/out.
 * Removed the two FILE pointers from ctors, since those are now set with
 * functions which are friends of BaseType.
 *
 * Revision 1.5  1994/12/16  15:17:57  dan
 * Removed inheritance from class CtorType, now directly inherits
 * from class BaseType.
 *
 * Revision 1.4  1994/11/22  14:05:54  jimg
 * Added code for data transmission to parts of the type hierarchy. Not
 * complete yet.
 * Fixed erros in type hierarchy headers (typos, incorrect comments, ...).
 *
 * Revision 1.3  1994/10/17  23:34:52  jimg
 * Added code to print_decl so that variable declarations are pretty
 * printed.
 * Added private mfunc duplicate().
 * Added ptr_duplicate().
 * Added Copy ctor, dtor and operator=.
 *
 * Revision 1.2  1994/09/23  14:43:39  jimg
 * Added mfunc check_semantics().
 * Added sanity checking on the variable lists (are the lists empty?).
 */

#ifndef _Function_h
#define _Function_h 1

#ifdef _GNUG_
#pragma interface
#endif

#include <rpc/types.h>
#include <netinet/in.h>
#include <rpc/xdr.h>

#include <SLList.h>

#include "BaseType.h"
#include "config_dap.h"
#ifdef TRACE_NEW
#include "trace_new.h"
#endif

/** This class holds the DODS Function data type.  This compound type
    contains an implied causal relationship between two sets of
    variables, the independent and the dependent.

    The Function data type can be thought of as a combination of two
    discrete Structure variables, with one complete Structure
    representing the independent variables and the other representing
    the dependent ones.  All the member functions of the Function have
    a parallel in the Structure class.

    Note that there is no naming distinction between independent and
    dependent variables.  There is no way to indicate that a variable
    is in one part or the other besides which list it appears in.
    This means that all the names within the function must be unique.

    As with Structure, you can refer to members of Function members
    with a ``.'' notation.  For example, if the Function has a member
    Structure called ``Ralph'' and Ralph has a member Int32 called
    ``age'', then you can refer to Ralph's age as ``Ralph.age''.
    
    The Function type should not be used by new programs.  Use the
    Sequence class object instead.

    @memo Holds the Function data type. 
    @see Structure */
class Function: public BaseType {
private:
    SLList<BaseTypePtr> _indep_vars;
    SLList<BaseTypePtr> _dep_vars;

    void _duplicate(const Function &s);

public:
  /** The Function constructor requires only the name of the variable
      to be created.  The name may be omitted, which will create a
      nameless variable.  This may be adequate for some applications. 
      
      @param n A String containing the name of the variable to be
      created.  

      @memo The Function constructor.
      */
    Function(const String &n = (char *)0);
  /** The Function copy constructor. */
    Function(const Function &rhs);
    virtual ~Function();

    const Function &operator=(const Function &rhs);
    virtual BaseType *ptr_duplicate() = 0;

    virtual int element_count(bool leaves = false);

    virtual void set_send_p(bool state);
    virtual void set_read_p(bool state);

    virtual BaseType *var(const String &name);

  /** Adds a variable to the indicated part of the Function.  All
      names of variables in a Function must be unique.  You cannot
      have independent and dependent variables with the same name.
      */
    virtual void add_var(BaseType *bt, Part part);

  /** Returns the index of the first independent variable. */
    Pix first_indep_var();
  /** Increments the given index to point to the next independent
      variable. */
    void next_indep_var(Pix &p);
  /** Returns a pointer to the indicated independent variable. */
    BaseType *indep_var(Pix p);

  /** Returns the index of the first dependent variable. */
    Pix first_dep_var();
  /** Increments the given index to point to the next dependent
      variable. */
    void next_dep_var(Pix &p);
  /** Returns a pointer to the indicated dependent variable. */
    BaseType *dep_var(Pix p);

    virtual unsigned int width();

    virtual bool serialize(const String &dataset, DDS &dds, XDR *sink,
			   bool ce_eval = true);
    virtual bool deserialize(XDR *source, DDS *dds, bool reuse = false);

    virtual bool read(const String &dataset, int &error) = 0;

    virtual unsigned int val2buf(void *val, bool reuse = false);
    virtual unsigned int buf2val(void **val);

    virtual void print_decl(ostream &os, String space = "    ",
			    bool print_semi = true,
			    bool constraint_info = false,
			    bool constraint = false);

    virtual void print_val(ostream &os, String space = "",
			   bool print_decl_p = true);

    virtual bool check_semantics(String &msg, bool all = false);
};

#endif
