
// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGH.  
//
// Authors:
//      jhrg,jimg       James Gallagher (jgallagher@gso.uri.edu)

// Implementation for List.
//
// jhrg 9/7/94

// $Log: List.cc,v $
// Revision 1.27  1996/12/02 23:17:48  jimg
// member member function now takes a DDS & in addition to the BaseType *. This
// makes it possible to use the new `filename' field of the DDS to pass the
// filename into member()'s use of ops().
//
// Revision 1.26  1996/08/13 18:33:37  jimg
// Added __unused__ to definition of char rcsid[].
// Fixed null() member function.
//
// Revision 1.25  1996/05/31 23:29:53  jimg
// Updated copyright notice.
//
// Revision 1.24  1996/05/29 22:08:41  jimg
// Made changes necessary to support CEs that return the value of a function
// instead of the value of a variable. This was done so that it would be
// possible to translate Sequences into Arrays without first reading the
// entire sequence over the network.
//
// Revision 1.23  1996/05/14 15:38:31  jimg
// These changes have already been checked in once before. However, I
// corrupted the source repository and restored it from a 5/9/96 backup
// tape. The previous version's log entry should cover the changes.
//
// Revision 1.22  1996/04/05 00:21:36  jimg
// Compiled with g++ -Wall and fixed various warnings.
//
// Revision 1.21  1996/04/04 18:20:16  jimg
// Merged changes from version 1.1.1.
// Removed old code from the CE functions for Lists (null, member, ...).
//
// Revision 1.20  1996/03/05 18:06:37  jimg
// Added member and null member functions and list_member and list_null
// functions (to be used as an interface by the CE evaluator).
//
// Revision 1.19  1996/02/02 00:31:09  jimg
// Merge changes for DODS-1.1.0 into DODS-2.x
//
// Revision 1.18  1995/11/22  22:30:59  jimg
// Modified so that the Vector class is now the parent class.
//
// Revision 1.17  1995/10/23  23:20:59  jimg
// Added _send_p and _read_p fields (and their accessors) along with the
// virtual mfuncs set_send_p() and set_read_p().
//
// Revision 1.16  1995/08/26  00:31:40  jimg
// Removed code enclosed in #ifdef NEVER #endif.
//
// Revision 1.15  1995/08/23  00:15:54  jimg
// Re-wrote List so that it uses the same basic algorithms as Array. The major
// difference is that the length of the List must be set manually (since it
// changes as the value of the list variable changes) whereas the length of an
// array can be computed from its declaration.
//
// Revision 1.14.2.2  1995/09/14 21:01:03  jimg
// Fixed _duplicate() so that it correctly uses BaseType's definition.
//
// Revision 1.14.2.1  1995/07/11  18:17:07  jimg
// Added cast of xdr_array (used in BaseType's constructor) to xdrproc_t.
//
// Revision 1.14  1995/07/09  21:29:02  jimg
// Added copyright notice.
//
// Revision 1.13  1995/05/10  13:45:23  jimg
// Changed the name of the configuration header file from `config.h' to
// `config_dap.h' so that other libraries could have header files which were
// installed in the DODS include directory without overwriting this one. Each
// config header should follow the convention config_<name>.h.
//
// Revision 1.12  1995/03/16  17:33:53  jimg
// The class List is still hopelessly broken... This is checked in only so
// that the rest of the library will compile.
//
// Revision 1.11  1995/03/04  15:30:07  jimg
// Fixed List so that it will compile - it still has major bugs.
// Modified Makefile.in so that test cases include List even though List
// won't work (but at least the test cases will link and run, so long
// as you don't try to do anything with Lists...
//
// Revision 1.10  1995/03/04  14:34:47  jimg
// Major modifications to the transmission and representation of values:
// Added card() virtual function which is true for classes that
// contain cardinal types (byte, int float, string).
// Changed the representation of Str from the C rep to a C++
// class represenation.
// Chnaged read_val and store_val so that they take and return
// types that are stored by the object (e.g., inthe case of Str
// an URL, read_val returns a C++ String object).
// Modified Array representations so that arrays of card()
// objects are just that - no more storing strings, ... as
// C would store them.
// Arrays of non cardinal types are arrays of the DODS objects (e.g.,
// an array of a structure is represented as an array of Structure
// objects).
//
// Revision 1.9  1995/02/10  02:22:57  jimg
// Added DBMALLOC includes and switch to code which uses malloc/free.
// Private and protected symbols now start with `_'.
// Added new accessors for name and type fields of BaseType; the old ones
// will be removed in a future release.
// Added the store_val() mfunc. It stores the given value in the object's
// internal buffer.
// Made both List and Str handle their values via pointers to memory.
// Fixed read_val().
// Made serialize/deserialize handle all malloc/free calls (even in those
// cases where xdr initiates the allocation).
// Fixed print_val().
//
// Revision 1.8  1995/01/19  20:05:23  jimg
// ptr_duplicate() mfunc is now abstract virtual.
// Array, ... Grid duplicate mfuncs were modified to take pointers, not
// referenves.
//
// Revision 1.7  1995/01/11  15:54:42  jimg
// Added modifications necessary for BaseType's static XDR pointers. This
// was mostly a name change from xdrin/out to _xdrin/out.
// Removed the two FILE pointers from ctors, since those are now set with
// functions which are friends of BaseType.
//
// Revision 1.6  1994/12/15  21:12:56  dan
// Fixed size() function to return cumulative size, in bytes, of list.
// Added length() function to return length of list.
// Fixed serialize() and deserialize().
//
// Revision 1.5  1994/12/15  15:33:16  jimg
// Added functions for data transmission.
// Added size and *buf members.
// Added accesser mfuncs for size memebr.
//
// Revision 1.4  1994/11/22  14:06:02  jimg
// Added code for data transmission to parts of the type hierarchy. Not
// complete yet.
// Fixed erros in type hierarchy headers (typos, incorrect comments, ...).
//
// Revision 1.3  1994/10/17  23:34:45  jimg
// Added code to print_decl so that variable declarations are pretty
// printed.
// Added private mfunc duplicate().
// Added ptr_duplicate().
// Added Copy ctor, dtor and operator=.
//
// Revision 1.2  1994/09/23  14:46:16  jimg
// Fixed some errors in comments.
//

#ifdef __GNUG__
#pragma implementation
#endif

#include "config_dap.h"

static char rcsid[] __unused__ = {"$Id: List.cc,v 1.27 1996/12/02 23:17:48 jimg Exp $"};

#include <assert.h>
#include <string.h>

#include "List.h"
#include "Error.h"		// Needed by parser.h
#include "util.h"
#include "parser.h"
#include "expr.tab.h"
#include "debug.h"

#ifdef TRACE_NEW
#include "trace_new.h"
#endif

// private

void
List::_duplicate(const List &l)
{
    Vector::_duplicate(l);
}

// public 

List::List(const String &n, BaseType *v) : Vector(n, v, dods_list_c)
{
}

List::List(const List &rhs)
{
    _duplicate(rhs);
}

List::~List()
{
}

const List &
List::operator=(const List &rhs)
{
    if (this == &rhs)
	return *this;

    _duplicate(rhs);

    return *this;
}

bool
List::member(BaseType *btp, DDS &dds)
{
    int len = length();

    for (int i = 0; i < len; ++i) {
	BaseType *a1 = var(i);	// get a pointer to the ith list element
	if (btp->ops(*a1, EQUAL, dds.filename()))
	    return true;
    }

    return false;
}

bool
List::null()
{
    return (length() == 0);
}



