// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)
    based on code by Marc Shapiro (shapiro@sor.inria.fr)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _StringXPlex_h
#ifdef __GNUG__
#pragma interface
#endif
#define _StringXPlex_h 1

#include "String.Plex.h"

class StringXPlex: public StringPlex
{
  StringIChunk*       ch;           // cached chunk

  void             make_initial_chunks(int up = 1);

  void             cache(int idx) const;
  void             cache(const String* p) const;

  String*             dopred(const String* p) const;
  String*             dosucc(const String* p) const;

  void             set_cache(const StringIChunk* t) const; // logically, 
                                               // not physically const
public:
                   StringXPlex();                 // set low = 0;
                                               // fence = 0;
                                               // csize = default

                   StringXPlex(int ch_size);      // low = 0; 
                                               // fence = 0;
                                               // csize = ch_size

                   StringXPlex(int lo,            // low = lo; 
                            int ch_size);      // fence=lo
                                               // csize = ch_size

                   StringXPlex(int lo,            // low = lo
                            int hi,            // fence = hi+1
                            const String  initval,// fill with initval,
                            int ch_size = 0);  // csize= ch_size
                                               // or fence-lo if 0

                   StringXPlex(const StringXPlex&);
  
  void             operator= (const StringXPlex&);

// virtuals


  String&             high_element ();
  String&             low_element ();

  const String&       high_element () const;
  const String&       low_element () const;

  Pix              first() const;
  Pix              last() const;
  void             prev(Pix& ptr) const;
  void             next(Pix& ptr) const;
  int              owns(Pix p) const;
  String&             operator () (Pix p);
  const String&       operator () (Pix p) const;

  int              low() const; 
  int              high() const;
  int              valid(int idx) const;
  void             prev(int& idx) const;
  void             next(int& x) const;
  String&             operator [] (int index);
  const String&       operator [] (int index) const;
    
  int              Pix_to_index(Pix p) const;
  Pix              index_to_Pix(int idx) const;    

  int              can_add_high() const;
  int              can_add_low() const;
  int              full() const;

  int              add_high(const String  elem);
  int              del_high ();
  int              add_low (const String  elem);
  int              del_low ();

  void             fill(const String  x);
  void             fill(const String  x, int from, int to);
  void             clear();
  void             reverse();
    
  int              OK () const; 

};


inline void StringXPlex::prev(int& idx) const
{
  --idx;
}

inline void StringXPlex::next(int& idx) const
{
  ++idx;
}

inline  int StringXPlex::full () const
{
  return 0;
}

inline int StringXPlex::can_add_high() const
{
  return 1;
}

inline int StringXPlex::can_add_low() const
{
  return 1;
}

inline  int StringXPlex::valid (int idx) const
{
  return idx >= lo && idx < fnc;
}

inline int StringXPlex::low() const
{
  return lo;
}

inline int StringXPlex::high() const
{
  return fnc - 1;
}

inline String& StringXPlex:: operator [] (int idx)
{
  if (!ch->actual_index(idx)) cache(idx);
  return *(ch->pointer_to(idx));
}

inline const String& StringXPlex:: operator [] (int idx) const
{
  if (!ch->actual_index(idx)) cache(idx);
  return *((const String*)(ch->pointer_to(idx)));
}

inline  String& StringXPlex::low_element ()
{
  if (empty()) index_error();
  return *(hd->pointer_to(lo));
}

inline  const String& StringXPlex::low_element () const
{
  if (empty()) index_error();
  return *((const String*)(hd->pointer_to(lo)));
}

inline  String& StringXPlex::high_element ()
{
  if (empty()) index_error();
  return *(tl()->pointer_to(fnc - 1));
}

inline const String& StringXPlex::high_element () const
{
  if (empty()) index_error();
  return *((const String*)(tl()->pointer_to(fnc - 1)));
}

inline  int StringXPlex::Pix_to_index(Pix px) const
{
  String* p = (String*)px;
  if (!ch->actual_pointer(p)) cache(p);
  return ch->index_of(p);
}

inline  Pix StringXPlex::index_to_Pix(int idx) const
{
  if (!ch->actual_index(idx)) cache(idx);
  return (Pix)(ch->pointer_to(idx));
}

inline Pix StringXPlex::first() const
{
  return Pix(hd->StringIChunk::first_pointer());
}

inline Pix StringXPlex::last() const
{
  return Pix(tl()->StringIChunk::last_pointer());
}

inline void StringXPlex::prev(Pix& p) const
{
  Pix q = Pix(ch->StringIChunk::pred((String*) p));
  p = (q == 0)? Pix(dopred((const String*) p)) : q;
}

inline void StringXPlex::next(Pix& p) const
{
  Pix q = Pix(ch->StringIChunk::succ((String*) p));
  p = (q == 0)? Pix(dosucc((const String*)p)) : q;
}

inline String& StringXPlex:: operator () (Pix p)
{
  return *((String*)p);
}

inline const String& StringXPlex:: operator () (Pix p) const
{
  return *((const String*)p);
}

inline void StringXPlex::set_cache(const StringIChunk* t) const
{
  ((StringXPlex*)(this))->ch = (StringIChunk*)t;
}

#endif
