// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)
    based on code by Marc Shapiro (shapiro@sor.inria.fr)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _BaseTypePtrXPlex_h
#ifdef __GNUG__
#pragma interface
#endif
#define _BaseTypePtrXPlex_h 1

#include "BTPlex.h"

class BaseTypePtrXPlex: public BaseTypePtrPlex
{
  BaseTypePtrIChunk*       ch;           // cached chunk

  void             make_initial_chunks(int up = 1);

  void             cache(int idx) const;
  void             cache(const BaseTypePtr* p) const;

  BaseTypePtr*             dopred(const BaseTypePtr* p) const;
  BaseTypePtr*             dosucc(const BaseTypePtr* p) const;

  void             set_cache(const BaseTypePtrIChunk* t) const; // logically, 
                                               // not physically const
public:
                   BaseTypePtrXPlex();                 // set low = 0;
                                               // fence = 0;
                                               // csize = default

                   BaseTypePtrXPlex(int ch_size);      // low = 0; 
                                               // fence = 0;
                                               // csize = ch_size

                   BaseTypePtrXPlex(int lo,            // low = lo; 
                            int ch_size);      // fence=lo
                                               // csize = ch_size

                   BaseTypePtrXPlex(int lo,            // low = lo
                            int hi,            // fence = hi+1
                            const BaseTypePtr  initval,// fill with initval,
                            int ch_size = 0);  // csize= ch_size
                                               // or fence-lo if 0

                   BaseTypePtrXPlex(const BaseTypePtrXPlex&);
  
  void             operator= (const BaseTypePtrXPlex&);

// virtuals


  BaseTypePtr&             high_element ();
  BaseTypePtr&             low_element ();

  const BaseTypePtr&       high_element () const;
  const BaseTypePtr&       low_element () const;

  Pix              first() const;
  Pix              last() const;
  void             prev(Pix& ptr) const;
  void             next(Pix& ptr) const;
  int              owns(Pix p) const;
  BaseTypePtr&             operator () (Pix p);
  const BaseTypePtr&       operator () (Pix p) const;

  int              low() const; 
  int              high() const;
  int              valid(int idx) const;
  void             prev(int& idx) const;
  void             next(int& x) const;
  BaseTypePtr&             operator [] (int index);
  const BaseTypePtr&       operator [] (int index) const;
    
  int              Pix_to_index(Pix p) const;
  Pix              index_to_Pix(int idx) const;    

  int              can_add_high() const;
  int              can_add_low() const;
  int              full() const;

  int              add_high(const BaseTypePtr  elem);
  int              del_high ();
  int              add_low (const BaseTypePtr  elem);
  int              del_low ();

  void             fill(const BaseTypePtr  x);
  void             fill(const BaseTypePtr  x, int from, int to);
  void             clear();
  void             reverse();
    
  int              OK () const; 

};


inline void BaseTypePtrXPlex::prev(int& idx) const
{
  --idx;
}

inline void BaseTypePtrXPlex::next(int& idx) const
{
  ++idx;
}

inline  int BaseTypePtrXPlex::full () const
{
  return 0;
}

inline int BaseTypePtrXPlex::can_add_high() const
{
  return 1;
}

inline int BaseTypePtrXPlex::can_add_low() const
{
  return 1;
}

inline  int BaseTypePtrXPlex::valid (int idx) const
{
  return idx >= lo && idx < fnc;
}

inline int BaseTypePtrXPlex::low() const
{
  return lo;
}

inline int BaseTypePtrXPlex::high() const
{
  return fnc - 1;
}

inline BaseTypePtr& BaseTypePtrXPlex:: operator [] (int idx)
{
  if (!ch->actual_index(idx)) cache(idx);
  return *(ch->pointer_to(idx));
}

inline const BaseTypePtr& BaseTypePtrXPlex:: operator [] (int idx) const
{
  if (!ch->actual_index(idx)) cache(idx);
  return *((const BaseTypePtr*)(ch->pointer_to(idx)));
}

inline  BaseTypePtr& BaseTypePtrXPlex::low_element ()
{
  if (empty()) index_error();
  return *(hd->pointer_to(lo));
}

inline  const BaseTypePtr& BaseTypePtrXPlex::low_element () const
{
  if (empty()) index_error();
  return *((const BaseTypePtr*)(hd->pointer_to(lo)));
}

inline  BaseTypePtr& BaseTypePtrXPlex::high_element ()
{
  if (empty()) index_error();
  return *(tl()->pointer_to(fnc - 1));
}

inline const BaseTypePtr& BaseTypePtrXPlex::high_element () const
{
  if (empty()) index_error();
  return *((const BaseTypePtr*)(tl()->pointer_to(fnc - 1)));
}

inline  int BaseTypePtrXPlex::Pix_to_index(Pix px) const
{
  BaseTypePtr* p = (BaseTypePtr*)px;
  if (!ch->actual_pointer(p)) cache(p);
  return ch->index_of(p);
}

inline  Pix BaseTypePtrXPlex::index_to_Pix(int idx) const
{
  if (!ch->actual_index(idx)) cache(idx);
  return (Pix)(ch->pointer_to(idx));
}

inline Pix BaseTypePtrXPlex::first() const
{
  return Pix(hd->BaseTypePtrIChunk::first_pointer());
}

inline Pix BaseTypePtrXPlex::last() const
{
  return Pix(tl()->BaseTypePtrIChunk::last_pointer());
}

inline void BaseTypePtrXPlex::prev(Pix& p) const
{
  Pix q = Pix(ch->BaseTypePtrIChunk::pred((BaseTypePtr*) p));
  p = (q == 0)? Pix(dopred((const BaseTypePtr*) p)) : q;
}

inline void BaseTypePtrXPlex::next(Pix& p) const
{
  Pix q = Pix(ch->BaseTypePtrIChunk::succ((BaseTypePtr*) p));
  p = (q == 0)? Pix(dosucc((const BaseTypePtr*)p)) : q;
}

inline BaseTypePtr& BaseTypePtrXPlex:: operator () (Pix p)
{
  return *((BaseTypePtr*)p);
}

inline const BaseTypePtr& BaseTypePtrXPlex:: operator () (Pix p) const
{
  return *((const BaseTypePtr*)p);
}

inline void BaseTypePtrXPlex::set_cache(const BaseTypePtrIChunk* t) const
{
  ((BaseTypePtrXPlex*)(this))->ch = (BaseTypePtrIChunk*)t;
}

#endif
