
// (c) COPYRIGHT URI/MIT 1995-1996
// Please read the full copyright statement in the file COPYRIGH.  
//
// Authors:
//      jhrg,jimg       James Gallagher (jgallagher@gso.uri.edu)

// Implementation for TestList. See TestByte.cc
//
// jhrg 1/12/95

// $Log: TestList.cc,v $
// Revision 1.19  1998/11/10 00:57:14  jimg
// Fixed errant delete [] (should have been plain delete).
//
// Revision 1.18  1996/12/02 18:21:19  jimg
// Added case for unit32 to ops() member functon.
//
// Revision 1.17  1996/08/13 20:50:45  jimg
// Changed definition of the read member function.
//
// Revision 1.16  1996/05/31 23:30:26  jimg
// Updated copyright notice.
//
// Revision 1.15  1996/05/22 18:05:26  jimg
// Merged files from the old netio directory into the dap directory.
// Removed the errmsg library from the software.
//
// Revision 1.14  1996/05/14 15:38:42  jimg
// These changes have already been checked in once before. However, I
// corrupted the source repository and restored it from a 5/9/96 backup
// tape. The previous version's log entry should cover the changes.
//
// Revision 1.13  1996/04/05 21:59:30  jimg
// Misc Changes for release 2.0.1 of the core software - for developers.
//
// Revision 1.12  1996/04/05 00:21:56  jimg
// Compiled with g++ -Wall and fixed various warnings.
//
// Revision 1.11  1996/03/05 18:51:30  jimg
// Fixed variable scoping problems in switch and for statements.
//
// Revision 1.10  1995/12/09  01:07:20  jimg
// Added changes so that relational operators will work properly for all the
// datatypes (including Sequences). The relational ops are evaluated in
// DDS::eval_constraint() after being parsed by DDS::parse_constraint().
//
// Revision 1.9  1995/12/06  19:55:25  jimg
// Changes read() member function from three arguments to two.
//
// Revision 1.8  1995/08/23  00:49:59  jimg
// Fixed the read() member function so that it works correctly for arrays/lists
// of Structure, ... types.
//
// Revision 1.7  1995/07/09  21:29:17  jimg
// Added copyright notice.
//
// Revision 1.6  1995/05/10  13:45:35  jimg
// Changed the name of the configuration header file from `config.h' to
// `config_dap.h' so that other libraries could have header files which were
// installed in the DODS include directory without overwriting this one. Each
// config header should follow the convention config_<name>.h.
//
// Revision 1.5  1995/03/16  17:33:52  jimg
// The class List is still hopelessly broken... This is checked in only so
// that the rest of the library will compile.
//
// Revision 1.4  1995/03/04  14:38:07  jimg
// Modified these so that they fit with the changes in the DAP classes.
//
// Revision 1.3  1995/02/10  02:33:45  jimg
// Modified Test<class>.h and .cc so that they used to new definitions of
// read_val().
// Modified the classes read() so that they are more in line with the
// class library's intended use in a real subclass set.
//
// Revision 1.2  1995/01/19  21:58:58  jimg
// Added read_val from dummy_read.cc to the sample set of sub-class
// implementations.
// Changed the declaration of readVal in BaseType so that it names the
// mfunc read_val (to be consistant with the other mfunc names).
// Removed the unnecessary duplicate declaration of the abstract virtual
// mfuncs read and (now) read_val from the classes Byte, ... Grid. The
// declaration in BaseType is sufficient along with the decl and definition
// in the *.cc,h files which contain the subclasses for Byte, ..., Grid.
//
// Revision 1.1  1995/01/19  20:20:49  jimg
// Created as an example of subclassing the class hierarchy rooted at
// BaseType.
//

#ifdef __GNUG__
#pragma implementation
#endif

#include "config_dap.h"

#include <assert.h>

#include "TestList.h"

#ifdef TRACE_NEW
#include "trace_new.h"
#endif

List *
NewList(const String &n, BaseType *v)
{
    return new TestList(n, v);
}

BaseType *
TestList::ptr_duplicate()
{
    return new TestList(*this);
}

TestList::TestList(const String &n, BaseType *v) : List(n, v)
{
}

TestList::~TestList()
{
}

// This `read mfunc' is just like the one for Array, except that it must set
// the List object's length member, since for a list, that value is a
// function of the instance and not of variable. For an array the size of
// thevariable must be declared (in the DDS). However, for a List the size
// will vary depending on the value of the variable (something that is *not*
// declared in the DDS).
//
// NB: we can never have lists of lists ... nor can we have arrays of arrays
// ... In DODS and array is an n-dimensional thing. Lists of lists are to
// amorphous for our translators.
//
// Returns: true.

bool 
TestList::read(const String &dataset, int &error)
{
    if (read_p())
	return true;

    // run read() on the contained variable to get, via the read() mfuncs
    // defined in the other Test classes, a value in the *contained* object.
    var()->read(dataset, error);

    unsigned int elem_wid = var()->width(); // size of an element

    set_length(7);		// this is a poor test; fix it

    unsigned int array_len = length();

    switch (var()->type()) {
      case dods_byte_c:
      case dods_int32_c:
      case dods_uint32_c:
      case dods_float64_c:
      case dods_str_c:
      case dods_url_c: {

	// String and Url are grouped with dods_byte, ... because val2buf works
	// for these types.

	char *tmp = new char[width()];
	void *elem_val = 0;	// NULL init gets read_val() to alloc space

	var()->buf2val(&elem_val); // internal buffer to ELEM_VAL

	for (unsigned i = 0; i < array_len; ++i)
	    memcpy(tmp + i * elem_wid, elem_val, elem_wid);


	val2buf(tmp);

	delete elem_val;	// alloced in buf2val()
	delete[] tmp;		// alloced above

	break;
      }

      case dods_array_c:
      case dods_structure_c:
      case dods_sequence_c:
      case dods_function_c:
      case dods_grid_c: {
	
	for (unsigned i = 0; i < array_len; ++i) {
	    BaseType *elem = var()->ptr_duplicate(); 
	    set_vec(i, elem);
	}

	break;
      }

      case dods_list_c:
      case dods_null_c:
      default:
	assert(false);
	cerr << "__FILE__:__LINE__ Bad data type" << endl;
	break;
    }

    set_read_p(true);

    return true;
}
