// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _BaseTypePtrVec_h
#ifdef __GNUG__
#pragma interface
#endif
#define _BaseTypePtrVec_h 1

#include <builtin.h>
#include "BaseTypedefs.h"

#ifndef _BaseTypePtr_typedefs
#define _BaseTypePtr_typedefs 1
typedef void (*BaseTypePtrProcedure)(BaseTypePtr );
typedef BaseTypePtr  (*BaseTypePtrMapper)(BaseTypePtr );
typedef BaseTypePtr  (*BaseTypePtrCombiner)(BaseTypePtr , BaseTypePtr );
typedef int  (*BaseTypePtrPredicate)(BaseTypePtr );
typedef int  (*BaseTypePtrComparator)(BaseTypePtr , BaseTypePtr );
#endif


class BaseTypePtrVec 
{
protected:      
  int                   len;
  BaseTypePtr                   *s;                  

                        BaseTypePtrVec(int l, BaseTypePtr* d);
public:
                        BaseTypePtrVec ();
                        BaseTypePtrVec (int l);
                        BaseTypePtrVec (int l, BaseTypePtr  fill_value);
                        BaseTypePtrVec (const BaseTypePtrVec&);
                        ~BaseTypePtrVec ();

  BaseTypePtrVec &              operator = (const BaseTypePtrVec & a);
  BaseTypePtrVec                at(int from = 0, int n = -1);

  int                   capacity() const;
  void                  resize(int newlen);                        

  BaseTypePtr&                  operator [] (int n);
  BaseTypePtr&                  elem(int n);

  friend BaseTypePtrVec         concat(BaseTypePtrVec & a, BaseTypePtrVec & b);
  friend BaseTypePtrVec         dods_map(BaseTypePtrMapper f, BaseTypePtrVec & a);
  friend BaseTypePtrVec         merge(BaseTypePtrVec & a, BaseTypePtrVec & b, BaseTypePtrComparator f);
  friend BaseTypePtrVec         combine(BaseTypePtrCombiner f, BaseTypePtrVec & a, BaseTypePtrVec & b);
  friend BaseTypePtrVec         reverse(BaseTypePtrVec & a);

  void                  reverse();
  void                  sort(BaseTypePtrComparator f);
  void                  fill(BaseTypePtr  val, int from = 0, int n = -1);

  void                  apply(BaseTypePtrProcedure f);
  BaseTypePtr                   reduce(BaseTypePtrCombiner f, BaseTypePtr  base);
  int                   index(BaseTypePtr  targ);

  friend int            operator == (BaseTypePtrVec& a, BaseTypePtrVec& b);
  friend int            operator != (BaseTypePtrVec& a, BaseTypePtrVec& b);

  void                  error(const char* msg);
  void                  range_error();
};

extern void default_BaseTypePtrVec_error_handler(const char*);
extern one_arg_error_handler_t BaseTypePtrVec_error_handler;

extern one_arg_error_handler_t 
        set_BaseTypePtrVec_error_handler(one_arg_error_handler_t f);


inline BaseTypePtrVec::BaseTypePtrVec()
{
  len = 0; s = 0;
}

inline BaseTypePtrVec::BaseTypePtrVec(int l)
{
  s = new BaseTypePtr [len = l];
}


inline BaseTypePtrVec::BaseTypePtrVec(int l, BaseTypePtr* d) :len(l), s(d) {}

inline BaseTypePtr& BaseTypePtrVec::operator [] (int n)
{
  if ((unsigned)n >= (unsigned)len)
    range_error();
  return s[n];
}

inline BaseTypePtr& BaseTypePtrVec::elem(int n)
{
  return s[n];
}

// Added the deletion of the BaseType pointers in THIS. jhrg 2/6/96

inline BaseTypePtrVec::~BaseTypePtrVec()
{
#ifdef NEVER
  for (int i = 0; i < len; ++i)
    delete elem(i);
#endif

  delete [] s;
}

inline int BaseTypePtrVec::capacity() const
{
  return len;
}



inline int operator != (BaseTypePtrVec& a, BaseTypePtrVec& b)
{
  return !(a == b);
}

#endif
