
# (c) COPYRIGHT 1996 URI/MIT
# Please read the full copyright statement in the file COPYRIGH.  
#
# Authors:
#      jhrg,jimg       James Gallagher (jgallagher@gso.uri.edu)

# These dialog boxes handle the display of various error messages.
#
# 6/18/96 jhrg

# $Log: error.tcl,v $
# Revision 1.3  1996/06/22 00:10:11  jimg
# Changed error window title.
# Moved progress code from dods_gui.tcl - the code is now used to
# initialize the GUI which is spread among several files.
#
# Revision 1.2  1996/06/19 23:24:50  jimg
# Added password dialog.
# Fixed up the error dialog; removed some unnecessary globals, set input focus.
#
# Revision 1.1  1996/06/19 20:52:11  jimg
# Start of the error processing Tcl code.
#

# Return the length of the longest word in MSG. Not currently used.

proc longest_word {msg} {
    set longest 0
    foreach w $msg {
	set len [string length $w]
	if {$len > $longest} {
	    set longest $len
	}
    }
    return $longest
}

# Display an error dialog with the TEXT and a single `Ok' button. Wait until
# the user clicks on the OK button and then send `OK' to stdout (so that
# expect can read something indicating that the user has dismissed the dialog
# box).

proc dialog {text {bitmap error}} {
    global font
    global err_geometry
    global err_done		;# Used in -command option; must be global

    if {![winfo exists .err]} {
	toplevel .err
	wm withdraw .err
	
	wm geometry .err $err_geometry
	wm title .err "transmission error"

	frame .err.f -relief groove -borderwidth 1
	label .err.f.icon -bitmap $bitmap
	message .err.f.msg -font $font -text $text -aspect 1000
	frame .err.bf -relief groove -borderwidth 1
	frame .err.bf.def -relief sunken -borderwidth 1
	button .err.bf.def.ok -text Ok -command { set err_done true }

	pack .err.f -side top -expand true -fill both -anchor n
	pack .err.f.icon -side left -padx 20 -pady 20 -anchor n
	pack .err.f.msg -side top -padx 5 -pady 5 -expand true -fill both \
		-anchor n
	pack .err.bf -side top -fill both -expand true -anchor s
	pack .err.bf.def -side top -padx 5 -pady 5 -anchor c
	pack .err.bf.def.ok -side top -padx 5 -pady 5

	bind .err <Return> { set err_done true }
    } else {
	# -width [expr [longest_word $text] + 10] 
	.err.f.msg configure -text $text
    }
    
    focus .err.bf.def.ok

    wm deiconify .err
    tkwait visibility .err
    grab set .err

    tkwait variable err_done
    wm withdraw .err
    grab release .err

    puts "OK"
}

# Prompt for a username and password. Return the two items in a list.

proc password {} {
    global font
    global pass_geometry
    global pass_button		;# Used in -command; must be global 

    if {![winfo exists .pass]} {
	toplevel .pass
	wm withdraw .pass

	wm geometry .pass $pass_geometry
	wm title .pass "Password"

	frame .pass.f -relief groove -borderwidth 1
	label .pass.f.icon -bitmap questhead
	frame .pass.f.lf
	label .pass.f.lf.user -text "Username:"
	label .pass.f.lf.pass -text "Password:"
	frame .pass.f.ef
	entry .pass.f.ef.user -font $font
	entry .pass.f.ef.pass -font $font -show "*"

	frame .pass.bf -relief groove -borderwidth 1
	frame .pass.bf.def -relief sunken -borderwidth 1
	button .pass.bf.def.ok -text "Ok" -command { set pass_button "OK" }
	button .pass.bf.cancel -text "Cancel" \
		-command {set pass_button "CANCEL" }

	pack .pass.f -side top -ipadx 10 -expand true -fill both 
	pack .pass.f.icon -side left -ipadx 20 -ipady 20 -anchor n
	pack .pass.f.lf -side left -anchor n -pady 10 
	pack .pass.f.lf.user -side top -anchor e -padx 2 -pady 2 -fill x \
		-expand true
	pack .pass.f.lf.pass -side top -anchor e -padx 2 -pady 2 -fill x \
		-expand true
	pack .pass.f.ef -side left -anchor n -pady 10
	pack .pass.f.ef.user -side top -anchor w -padx 2 -pady 2 -fill x \
		-expand true
	pack .pass.f.ef.pass -side top -anchor w -padx 2 -pady 2 -fill x \
		-expand true
	
	pack .pass.bf -side top -fill both -expand true -anchor s
	pack .pass.bf.def -side left -padx 5 -pady 5 -expand true
	pack .pass.bf.def.ok -side top -padx 5 -pady 5
	pack .pass.bf.cancel -side left -padx 5 -pady 5 -expand true

	bind .pass <Return> { set pass_button "OK" }
    } else {
	# If already built, erase the prvious values.
	.pass.f.ef.user delete 0 end
	.pass.f.ef.pass delete 0 end
    }

    focus .pass.f.ef.user 	;# Move to the first window

    wm deiconify .pass
    tkwait visibility .pass
    grab set .pass

    tkwait variable pass_button
    wm withdraw .pass
    grab release .pass

    puts $pass_button
    if {$pass_button == "OK"} {
	puts -nonewline [.pass.f.ef.user get]
	puts -nonewline " "
	puts [.pass.f.ef.pass get]
    }
}

proc simple_error {msg} {
    tk_dialog .tmp "Error" $msg error 0 "Ok"
}
