%
%  URI 1.2km Great Lakes SST fronts

GetFunctionName = 'getfronts';
URL_m_File = 'getfrontsurl';
Directory = 'none';
CatalogServer = 'none';

Server = str2mat('sst','sstgrad','sstgraddir');
Server2 = 'http://fronts.gso.uri.edu/cgi-bin/nph-jg/glkfronts';


LonRange = [-92.20 -76.10];
LatRange = [39.42 50.78];
TimeRange = [1985.0 1995.0];
DepthRange = [0 0];
Resolution = 1.236;         % avg resolution

%DataNull = [];
DataScale = [NaN NaN; NaN NaN; NaN NaN; NaN NaN; 0 1; 0 1; 0 1;];
DataRange = [NaN NaN; -92.20 -76.10; 39.42 50.78; NaN NaN; NaN NaN; NaN NaN; NaN NaN];

DodsName1 = str2mat('time','inlat','ixlat','inlon','ixlon','lat','lon');
DodsName  = str2mat('sst','sstgrad','sstgraddir');

% Name for this dataset.
DataName = 'Great Lakes SST Fronts';       
LongitudeName = 'lon';
LatitudeName = 'lat';
TimeName = 'time';           %correspond to 'time' in dataset
DepthName = '';

SelectableVariables = str2mat('Sea_Temp','Sea_Temp_Grad','Sea_Temp_Grad_Direction');

Comments1 = sprintf('%s\n', ...
'                    URI 1.2km Great Lakes SST fronts', ...
'     ', ...
'    The sea surface temperature (SST) fronts in this dataset are objectively', ...
'    located by application of an automated edge detection algorithm to SST', ...
'    images. ', ...
'    ', ...
'    The images cover the area from 34.68N to 45.72N and from 77.53W to 63.07W', ...
'    with a resolution of approximately 1.2 km/pixel, and are available for the', ...
'    period 1985 - 1996. (To view these SST images via this interface select the', ...
'     "Hatteras to Nova Scotia AVHRR" data set.)', ...
'    ', ...
'    Prior to edge detection, cloud-contaminated pixels are masked in the images', ...
'    using a declouding algorithm developed at the University of Rhode island. ', ...
'    For each SST image, this dataset provides location, SST, SST gradient and SST ', ...
'    gradient direction information for each pixel that was identified as a frontal ', ...
'    pixel.',...
'    ', ...
'    References ', ...
'    ', ...
'    Cayula, J-F., P. Cornillon, R. Holyer and S. Peckinpaugh. Comparative Study of Two ',...
'    Recent Edge Detection Algorithms Designed to Process Sea Surface Temperature Fields. ',...
'    IEEE Trans. on Geo. Rem. Sens., 29:175-177, 1991.',...
'    ',...
'    Cayula, J-F. and P. Cornillon. Edge detection algorithm for SST images. J. Atmos. ',...
'    Oceanic Technol., Vol. 9, No. 1, 67-80, 1992 ', ...
'    ', ...
'    Cayula, J-F. Multi-image edge detection for SST images. J. Atmos. Oceanic Technol.,',...
'    Vol. 12, No. 4, 821-829, 1995 ', ...
'    ', ...
'    Cayula, J-F. and P. Cornillon. Cloud Detection from a Sequence of SST Images. Remote',...
'    Sens. Environ, 55:80-88, 1996. ',...
'    ', ...'
'    Ullman, D., J. Brown, P. Cornillon, and T. Mavor, Surface Temperature Fronts in the ',...
'    Great Lakes, Journal of Great Lakes Research, 1998 (in press).');

Comments2 = sprintf('%s\n', ...
'   ', ...
'    Time_Period_of_Content = ', ...
'         Beginning_Date: 1985 ', ...
'         Ending_Date: 1995 ', ...
'    Spatial_Domain = ', ...
'         Bounding Coordinates: ', ...
'              West Bounding Coordinate: 92.196 W ', ...
'              East Bounding Coordinate: 76.104 W ', ...
'              South Bounding Coordinate: 39.422 N ', ...
'              North Bounding Coordinate: 50.778 N ', ...
'    Units:  Once converted from digital counts ==> Degrees Centigrade ', ...
'    Spatial Resolution: 1.2 km',...
'    Thermal Resolution: .125 deg. C ', ...
'   ', ...
'    Point_of_Contact = ', ...
'         Address: ', ...
'               Peter Cornillon ', ...
'               Graduate School of Oceanography ', ...
'               University of Rhode Island ', ...
'               South Ferry Road ', ...
'               Narragansett, Rhode Island  02882 ', ...
'               USA ', ...
'         Electronic Mail Address:INTERNET > support@dods.gso.uri.edu ');

Comments = [Comments1 Comments2];