%
%               FILE DESCRIBING THE URI HIGH RESOLUTION SST DATA SET
%                              Cape Hatteras to Nova Scotia

% FUNCTIONS -- REQUIRED
GetFunctionName = 'getrectg';
% FUNCTIONS -- OPTIONAL
Cat_m_File = 'urieccat';
URL_m_File = 'htnurl';

% VARIABLES -- OPTIONAL
Server = 'http://fronts.gso.uri.edu/cgi-bin/nph-dsp/htn_sst_decloud/';
CatalogServer = 'http://dods.gso.uri.edu/cgi-bin/nph-jg/htn_sst_decloud';
Nlon = 1024;
Nlat = 1024;

% VARIABLES -- REQUIRED
LonRange = [-77.5397 -63.0603];
LatRange = [45.7296 34.6704];
TimeRange = [1985 1995.99943];
DepthRange = [0 0];
Resolution = 1.236;                  % avg res.

DataName = 'Hatteras to N.S. AVHRR'; % Name for this dataset.
LongitudeName = 'lon';
LatitudeName = 'lat';
TimeName = '';
DepthName = '';
SelectableVariables = str2mat('Sea_Temp');
DodsName = 'dsp_band_1';

% note: DataNull is applied *before* scaling
DataNull = [NaN NaN NaN NaN 0];
DataScale = [NaN NaN; NaN NaN; NaN NaN; NaN NaN; -2.0  0.125];
DataRange = [NaN NaN; NaN NaN; 45.7242 34.6758; NaN NaN; ...
  0.0 32.0];

% COMMENTS
Comments1 = sprintf('%s\n', ...
'                  URI 1.2km Hatteras to Nova Scotia SST fields', ...
'     ', ...
'    This is a subset of the University of Rhode Island, Graduate School of ', ...
'    Oceanographys archive of sea surface temperature satellite images. These', ...
'    images are processed Advanced Very High Resolution Radiometer (AVHRR)', ...
'    data from NOAA satellites. ', ...
'    ', ...
'    The archive contains up to 5 images per day. We are reprocessing all', ...
'    of the data in our archive which covers the period from April 1979 to', ...
'    the present.  We began the reprocessing effort with data in the late', ...
'    1980''s and are processing forward and backward in time. The NASA/NOAA', ...
'    Pathfinder SST algorithm is being used for the reprocessing. Each of the ', ...
'    1024 by 1024 pixel images in the archive covers the area from latitude 34.68N', ...
'    longitude 77.53W to latitude 45.72N longitude 63.07W using a cylindrical ', ...
'    equirectangular map projection with a resolution of approximately 1.2 km/pixel.',...
'    Temperature data values are stored as 8 bit unsigned integer values (digital', ...
'    counts 0 - 255). The temperature (T) in degrees centigrade for a digital', ...
'    count of D is given by the  equation: ',...
'     ', ...
'                                        T = 0.125 * D - 2',...
'   ', ... 
'    Digital count values 0-3 are reserved for graphics, a digital count of 4 ', ...
'    represents clouds.  Valid temperature values are in the range 0.625 - 31.875 deg. C', ...
'    with 0.0-0.375 not used and 0.5 corresponding to clouds.', ...
'    Scaling can be turned off by setting all values of DataScale to NaN.');
Comments2 = sprintf('%s\n', ...
'   ', ...
'    Time_Period_of_Content = ', ...
'         Beginning_Date: Jan. 1, 1985 ', ...
'         Ending_Date: Dec. 31, 1995 ', ...
'    Spatial_Domain = ', ...
'         Bounding Coordinates: ', ...
'              West Bounding Coordinate: 77.5326 W ', ...
'              East Bounding Coordinate: 63.0674 W ', ...
'              South Bounding Coordinate: 34.6758 N ', ...
'              North Bounding Coordinate: 45.7242 N ', ...
'    Units:  Once converted from digital counts ==> Degrees Centigrade ', ...
'    Spatial Resolution: 1.2 km',...
'    Thermal Resolution: .125 deg. C ', ...
'   ', ...
'    Point_of_Contact = ', ...
'         Address: ', ...
'               Peter Cornillon ', ...
'               Graduate School of Oceanography ', ...
'               University of Rhode Island ', ...
'               South Ferry Road ', ...
'               Narragansett, Rhode Island  02882 ', ...
'               USA ', ...
'         Electronic Mail Address:INTERNET > support@dods.gso.uri.edu ');

Comments = [Comments1 Comments2];

