%
%               FILE DESCRIBING THE URI PREVU SST DATA SET

% The preceding empty line is important.
%
% $Id: prevu.m,v 1.5 1998/09/14 01:32:57 dbyrne Exp $

% $Log: prevu.m,v $
% Revision 1.5  1998/09/14 01:32:57  dbyrne
% Elminated an inconsistency whereby the depth variable was not scaled
% and had no DataNull etc.  Added check for empty variables to plotscript.m
%
% Revision 1.10  1998/09/13 19:33:25  root
% updated DataNull &c.
%
% Revision 1.9  1998/09/13 19:21:25  root
% Eliminated an inconsistency in DataScale, DataRange, and DataNull, that
% Depth was not included!
%
% Revision 1.8  1998/09/11 18:01:55  dbyrne
% undid latitude range swap!
%
% Revision 1.7  1998/09/11 08:16:10  dbyrne
% Longitude ranges were messed up in every archive.m file except the
% two I wrote myself.  To eliminate future confusion, 'LLon' 'Llat',
% 'Rlon' and 'Ulat' etc are being eliminated in favor of two-component
% vectors LonRange LatRange TimeRange DepthRange.
%
% Revision 1.6  1998/09/09 20:12:18  dbyrne
% Changing variable names to stay within 14 characters.
%
% Revision 1.5  1998/09/09 16:10:28  dbyrne
% Fixed numerous small bugs.
%
% Revision 1.4  1998/09/09 07:57:39  dbyrne
% replaced Data_Scale with DataScale, Data_Null with DataNull, and Data_Range
% with DataRange for consistency with other variables in the archive.m files.
%
% Revision 1.3  1998/09/09 07:40:23  dbyrne
% Replaced getrectangular with getrectg
%
% Revision 1.2  1998/09/09 07:34:23  dbyrne
% Eliminating ReturnedVariables
%
% Revision 1.1  1998/05/17 14:18:11  dbyrne
% *** empty log message ***
%
% Revision 1.1.1.1  1997/09/22 14:13:55  tom
% Imported Matlab GUI sources to CVS
%

% FUNCTIONS -- REQUIRED
GetFunctionName = 'getrectg';
% FUNCTIONS -- OPTIONAL
Cat_m_File = 'urieccat';
URL_m_File = 'prevuurl';

% VARIABLES -- OPTIONAL
Server = 'http://dods.gso.uri.edu/cgi-bin/nph-dsp/avhrr/';
CatalogServer = 'http://dods.gso.uri.edu/cgi-bin/nph-jg/avhrr';
Nlon = 1024;
Nlat = 1024;

% VARIABLES -- REQUIRED
LonRange = [-96.2525 -33.7475];
LatRange = [60.6000 9.4000];
TimeRange = [1979.276 1996.99966];
DepthRange = [0 0];
Resolution = 5.5446;

DataName = 'URI 5.5km Browse AVHRR SST';
LongitudeName = 'lon';
LatitudeName = 'lat';
TimeName = '';
DepthName = '';
SelectableVariables = str2mat('Sea_Temp');
DodsName = 'dsp_band_1';

% note: DataNull is applied *before* scaling
DataNull = [NaN NaN NaN NaN 0];
DataScale = [NaN NaN; NaN NaN; NaN NaN; NaN NaN;  0.0  0.125];
DataRange = [NaN NaN; NaN NaN; NaN NaN; NaN NaN;  0.625 31.875];

% COMMENTS
Comments1 = sprintf('%s\n', ...
'                           URI 5.5km Browse AVHRR SST', ...
'     ', ...
'    This is the University of Rhode Island, Graduate School of Oceanographys ', ...
'    archive of sea surface temperature satellite images. These images are ', ...
'    processed Advanced Very High Resolution Radiometer (AVHRR) data from NOAA ', ...
'    satellites. ', ...
'    ', ...
'    The archive contains in excess of 25,000 images from April 1979 to the ', ...
'    present.  New images are added to the archive every day at 10:30 GMT. During ', ...
'    the  first two years of the archive holdings, coverage is quite spotty, but ', ...
'    subsequent years generally have several images for each day.  Each of the ', ...
'    1024 by 1024 pixel images in the archive covers the area from latitude 60.575N ', ...
'    longitude 96.222W to latitude 9.399N longitude 33.768W using a cylindrical ', ...
'    equirectangular map projection with a resolution of approximately 5km/pixel.',...
'    The SST values in this dataset were determined using the operational MCSST ',...
'    algorithm.  Temperature data values are stored as 8 bit unsigned integer values',...
'    (digital counts 0 - 255). The data are stored in a 1024x1024 grid with a grid ',...
'    lat/lon resolution of 5.5x5.5 km at the image center. The grid origin, ',...
'    (indices (1,1)) corresponds to lat/lon of (60.575,-96.222). The temperature (T)',...
'    in degrees centigrade for a digital count of D is given by the  equation: ',...
'     ', ...
'                                        T = 0.125 * D - 2 ',...
'   ', ... 
'    Digital count values 0-3 are reserved for graphics, a digital count of 4 ', ...
'    represent clouds.  Valid temperature values are in the range 0.625-31.875 deg. C', ...
'    with 0.5 corresponding to clouds.',...
'    Scaling can be turned off by setting all values of DataScale to NaN.',...
'   ', ... 
'    References: ', ...
'   ', ... 
'    Multi-channel improvements to satellite-derived global sea surface ', ...
'        temperatures.  E.P. McClain, W.G. Pichel, C.C. Walton, Z. Ahmad and J. ', ...
'        Sutton.  Advances in Space Research, vol. 2,  no. 6, pp43-47, 1983.', ...
'   ', ...
'    NOAA/NESDIS, 1982: Coefficients presented at the 32nd SST Research Panel ', ...
'        Meeting, Suitland, Maryland.', ...
'   ', ...
'    NOAA/NESDIS, 1985: Coefficients presented at the 48th SST Research Panel ', ...
'        Meeting, Suitland, Maryland.');

Comments2 = sprintf('%s\n', ...
'   ', ...
'    Time_Period_of_Content = ', ...
'         Beginning_Date: Apr. 1979 ', ...
'         Ending_Date: Dec. 1996 ', ...
'    Spatial_Domain = ', ...
'         Bounding Coordinates: ', ...
'              West Bounding Coordinate:-96.222 ', ...
'              East Bounding Coordinate:-33.768 ', ...
'              South Bounding Coordinate: 9.425 ', ...
'              North Bounding Coordinate: 60.575 ', ...
'    Units:  Once converted from digital counts ==> Degrees Centigrade ', ...
'    Spatial Resolution: 1.2 km',...
'    Thermal Resolution: .125 deg. C ', ...
'   ', ...
'    Point_of_Contact = ', ...
'         Address: ', ...
'               Peter Cornillon ', ...
'               Graduate School of Oceanography ', ...
'               University of Rhode Island ', ...
'               South Ferry Road ', ...
'               Narragansett, Rhode Island  02882 ', ...
'               USA ', ...
'         Electronic Mail Address:INTERNET > support@dods.gso.uri.edu ');

Comments = [Comments1 Comments2];
